import 'package:flutter/material.dart';
import 'package:flutter_flicker/configuration/configuration.dart';
import 'package:intl/intl.dart';
import 'decorator_utils.dart';

enum FlickerDateMode { Date, TimeOfDay, DateAndTime }

class FlickerDateInput extends StatefulWidget {
  FlickerDateInput({
    Key? key,
    this.initialDateValue,
    this.labelText,
    this.uppercaseLabelAndHint = true,
    this.onDateChanged,
    this.onTimeOfDayChanged,
    this.mode = FlickerDateMode.Date,
    this.readonly = false,
    this.initialTimeOfDayValue,
  }) : super(key: key);

  final DateTime? initialDateValue;
  final TimeOfDay? initialTimeOfDayValue;
  final String? labelText;
  final bool uppercaseLabelAndHint;
  final ValueChanged<DateTime?>? onDateChanged;
  final ValueChanged<TimeOfDay>? onTimeOfDayChanged;
  final bool readonly;
  final FlickerDateMode mode;

  @override
  _FlickerDateInputState createState() => _FlickerDateInputState();
}

class _FlickerDateInputState extends State<FlickerDateInput> {
  final TextEditingController _controller = TextEditingController();
  DateTime? _currentDate;

  @override
  void initState() {
    if (widget.initialDateValue != null) {
      _currentDate = widget.initialDateValue!.toLocal();
    }

    _currentDate ??= DateTime.now();

    if (widget.initialTimeOfDayValue != null) {
      _currentDate = DateTime(
        _currentDate!.year,
        _currentDate!.month,
        _currentDate!.day,
        widget.initialTimeOfDayValue!.hour,
        widget.initialTimeOfDayValue!.minute,
        _currentDate!.second,
        _currentDate!.millisecond,
        _currentDate!.microsecond,
      );
    }
    _controller.text = getValue(_currentDate);

    _controller.addListener(() {
      fireOnChanged(_controller.text);
    });

    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return TextFormField(
      readOnly: widget.readonly,
      decoration: InputDecoration(
        labelText: DecoratorUtils.buildLabelText(widget.labelText, uppercase: widget.uppercaseLabelAndHint),
        hintText: DecoratorUtils.buildHintText(null, labelText: widget.labelText, uppercase: widget.uppercaseLabelAndHint),
        suffix: Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.end,
          children: [
            if (widget.mode == FlickerDateMode.Date ||
                widget.mode == FlickerDateMode.DateAndTime)
              Builder(
                builder: (context) => MouseRegion(
                  cursor: SystemMouseCursors.click,
                  child: GestureDetector(
                    child: Icon(
                      Icons.date_range_rounded,
                      color: FlickerConfiguration.colors.inputControlBorder,
                      size: 21,
                    ),
                    onTap: () async {
                      final picked = await showDatePicker(
                          context: context,
                          initialDate: _currentDate!,
                          initialDatePickerMode: DatePickerMode.day,
                          firstDate: DateTime(1900),
                          lastDate: DateTime(2030));
                      if (picked != null) {
                        setState(() {
                          _currentDate = DateTime(
                            picked.year,
                            picked.month,
                            picked.day,
                            _currentDate!.hour,
                            _currentDate!.minute,
                            _currentDate!.second,
                            _currentDate!.microsecond,
                            _currentDate!.microsecond,
                          );
                          _controller.text = getValue(_currentDate);
                          fireOnChanged(_controller.text);
                        });
                      }
                    },
                  ),
                ),
              ),
            if (widget.mode == FlickerDateMode.DateAndTime)
              SizedBox(
                width: 4.0,
              ),
            if (widget.mode == FlickerDateMode.DateAndTime ||
                widget.mode == FlickerDateMode.TimeOfDay)
              Builder(
                builder: (context) => MouseRegion(
                  cursor: SystemMouseCursors.click,
                  child: GestureDetector(
                    child: Icon(
                      Icons.access_time_rounded,
                      color: FlickerConfiguration.colors.inputControlBorder,
                      size: 21,
                    ),
                    onTap: () async {
                      final picked = await showTimePicker(
                        context: context,
                        initialTime: TimeOfDay(
                          hour: _currentDate!.hour,
                          minute: _currentDate!.minute,
                        ),
                      );
                      if (picked != null) {
                        setState(() {
                          _currentDate = DateTime(
                            _currentDate!.year,
                            _currentDate!.month,
                            _currentDate!.day,
                            picked.hour,
                            picked.minute,
                            _currentDate!.second,
                            _currentDate!.millisecond,
                            _currentDate!.microsecond,
                          );
                          _controller.text = getValue(_currentDate);
                          fireOnChanged(_controller.text);
                        });
                      }
                    },
                  ),
                ),
              ),
            SizedBox(
              width: 4.0,
            ),
          ],
        ),
      ),
      controller: _controller,
    );
  }

  DateTime fireOnChanged(String val) {
    if (val.isEmpty) widget.onDateChanged!(null);
    var dateVal = DateTime.tryParse(val);
    dateVal ??= widget.initialDateValue;
    if (widget.onDateChanged != null) {
      widget.onDateChanged!(dateVal);
    }

    if (widget.onTimeOfDayChanged != null) {
      widget.onTimeOfDayChanged!(
        TimeOfDay(hour: dateVal!.hour, minute: dateVal.minute),
      );
    }
    return dateVal!;
  }

  String getValue(DateTime? dateValue) {
    if (dateValue == null) return '';

    switch (widget.mode) {
      case FlickerDateMode.Date:
        return DateFormat('yyyy-MM-dd').format(dateValue);
      case FlickerDateMode.TimeOfDay:
        return DateFormat('HH:mm').format(dateValue);
      case FlickerDateMode.DateAndTime:
        return DateFormat('yyyy-MM-dd HH:mm').format(dateValue);
      default:
        return '';
    }
  }
}
